@@ -10,17 +10,20 @@
 
 use super::{
     central_delegate::{CentralDelegate, CentralDelegateEvent},
-    framework::{cb, ns},
+    framework::{
+        cb::{self, CBManagerAuthorization},
+        ns,
+    },
     future::{BtlePlugFuture, BtlePlugFutureStateShared},
-    utils::{CoreBluetoothUtils, NSStringUtils},
+    utils::{core_bluetooth::cbuuid_to_uuid, nsstring::nsstring_to_string, nsuuid_to_uuid},
 };
 use crate::api::{CharPropFlags, Characteristic, WriteType};
-use async_std::task;
+use crate::Error;
 use futures::channel::mpsc::{self, Receiver, Sender};
 use futures::select;
 use futures::sink::SinkExt;
 use futures::stream::{Fuse, StreamExt};
-use log::{error, info, trace};
+use log::{error, info, trace, warn};
 use objc::{
     rc::StrongPtr,
     runtime::{Object, YES},
@@ -30,11 +33,10 @@ use std::{
     fmt::{self, Debug, Formatter},
     ops::Deref,
     os::raw::c_uint,
-    str::FromStr,
     thread,
 };
+use tokio::runtime;
 use uuid::Uuid;
-use CoreBluetoothUtils::cbuuid_to_uuid;
 
 struct CBCharacteristic {
     pub characteristic: StrongPtr,
@@ -77,7 +79,7 @@ impl CBCharacteristic {
 
     fn form_flags(characteristic: *mut Object) -> CharPropFlags {
         let flags = cb::characteristic_properties(characteristic);
-        let mut v = CharPropFlags::new();
+        let mut v = CharPropFlags::default();
         if (flags & cb::CHARACTERISTICPROPERTY_BROADCAST) != 0 {
             v |= CharPropFlags::BROADCAST;
         }
@@ -99,7 +101,7 @@ impl CBCharacteristic {
         if (flags & cb::CHARACTERISTICPROPERTY_AUTHENTICATEDSIGNEDWRITES) != 0 {
             v |= CharPropFlags::AUTHENTICATED_SIGNED_WRITES;
         }
-        info!("Flags: {:?}", v);
+        trace!("Flags: {:?}", v);
         v
     }
 }
@@ -186,10 +188,6 @@ impl CBPeripheral {
             let mut char_set = BTreeSet::new();
             for (&uuid, c) in &self.characteristics {
                 let char = Characteristic {
-                    // We can't get handles on macOS, just set them to 0.
-                    start_handle: 0,
-                    end_handle: 0,
-                    value_handle: 0,
                     uuid,
                     properties: c.properties,
                 };
@@ -300,9 +298,10 @@ impl CoreBluetoothInternal {
         manufacturer_id: u16,
         manufacturer_data: Vec<u8>,
     ) {
-        info!(
+        trace!(
             "Got manufacturer data advertisement! {}: {:?}",
-            manufacturer_id, manufacturer_data
+            manufacturer_id,
+            manufacturer_data
         );
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             if let Err(e) = p
@@ -323,7 +322,7 @@ impl CoreBluetoothInternal {
         peripheral_uuid: Uuid,
         service_data: HashMap<Uuid, Vec<u8>>,
     ) {
-        info!("Got service data advertisement! {:?}", service_data);
+        trace!("Got service data advertisement! {:?}", service_data);
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             if let Err(e) = p
                 .event_sender
@@ -336,7 +335,7 @@ impl CoreBluetoothInternal {
     }
 
     async fn on_services(&mut self, peripheral_uuid: Uuid, services: Vec<Uuid>) {
-        info!("Got service advertisement! {:?}", services);
+        trace!("Got service advertisement! {:?}", services);
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             if let Err(e) = p
                 .event_sender
@@ -349,9 +348,8 @@ impl CoreBluetoothInternal {
     }
 
     async fn on_discovered_peripheral(&mut self, peripheral: StrongPtr) {
-        let uuid_nsstring = ns::uuid_uuidstring(cb::peer_identifier(*peripheral));
-        let uuid = Uuid::from_str(&NSStringUtils::string_to_string(uuid_nsstring)).unwrap();
-        let name = NSStringUtils::string_to_maybe_string(cb::peripheral_name(*peripheral));
+        let uuid = nsuuid_to_uuid(cb::peer_identifier(*peripheral));
+        let name = nsstring_to_string(cb::peripheral_name(*peripheral));
         if self.peripherals.contains_key(&uuid) {
             if let Some(name) = name {
                 self.dispatch_event(CoreBluetoothEvent::DeviceUpdated(uuid, name))
@@ -376,9 +374,9 @@ impl CoreBluetoothInternal {
         peripheral_uuid: Uuid,
         service_map: HashMap<Uuid, StrongPtr>,
     ) {
-        info!("Found services!");
+        trace!("Found services!");
         for id in service_map.keys() {
-            info!("{}", id);
+            trace!("{}", id);
         }
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             p.set_services(service_map);
@@ -390,9 +388,9 @@ impl CoreBluetoothInternal {
         peripheral_uuid: Uuid,
         char_map: HashMap<Uuid, StrongPtr>,
     ) {
-        info!("Found chars!");
+        trace!("Found chars!");
         for id in char_map.keys() {
-            info!("{}", id);
+            trace!("{}", id);
         }
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             p.set_characteristics(char_map);
@@ -448,20 +446,18 @@ impl CoreBluetoothInternal {
                 // we're trying to do a read, we'll have a future we can
                 // fulfill. Otherwise, just treat the returned value as a
                 // notification and use the event system.
-                if c.read_future_state.len() > 0 {
+                if !c.read_future_state.is_empty() {
                     let state = c.read_future_state.pop_back().unwrap();
                     state
                         .lock()
                         .unwrap()
                         .set_reply(CoreBluetoothReply::ReadResult(data_clone));
-                } else {
-                    if let Err(e) = p
-                        .event_sender
-                        .send(CBPeripheralEvent::Notification(characteristic_uuid, data))
-                        .await
-                    {
-                        error!("Error sending notification event: {}", e);
-                    }
+                } else if let Err(e) = p
+                    .event_sender
+                    .send(CBPeripheralEvent::Notification(characteristic_uuid, data))
+                    .await
+                {
+                    error!("Error sending notification event: {}", e);
                 }
             }
         }
@@ -496,7 +492,7 @@ impl CoreBluetoothInternal {
     ) {
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                info!("Writing value! With kind {:?}", kind);
+                trace!("Writing value! With kind {:?}", kind);
                 cb::peripheral_writevalue_forcharacteristic(
                     *p.peripheral,
                     ns::data(data.as_ptr(), data.len() as c_uint),
@@ -506,7 +502,13 @@ impl CoreBluetoothInternal {
                         WriteType::WithoutResponse => 1,
                     },
                 );
-                c.write_future_state.push_front(fut);
+                // WriteWithoutResponse does not call the corebluetooth
+                // callback, it just always succeeds silently.
+                if kind == WriteType::WithoutResponse {
+                    fut.lock().unwrap().set_reply(CoreBluetoothReply::Ok);
+                } else {
+                    c.write_future_state.push_front(fut);
+                }
             }
         }
     }
@@ -519,7 +521,7 @@ impl CoreBluetoothInternal {
     ) {
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                info!("Reading value!");
+                trace!("Reading value!");
                 cb::peripheral_readvalue_forcharacteristic(*p.peripheral, *c.characteristic);
                 c.read_future_state.push_front(fut);
             }
@@ -534,7 +536,7 @@ impl CoreBluetoothInternal {
     ) {
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                info!("Setting subscribe!");
+                trace!("Setting subscribe!");
                 cb::peripheral_setnotifyvalue_forcharacteristic(
                     *p.peripheral,
                     objc::runtime::YES,
@@ -553,7 +555,7 @@ impl CoreBluetoothInternal {
     ) {
         if let Some(p) = self.peripherals.get_mut(&peripheral_uuid) {
             if let Some(c) = p.characteristics.get_mut(&characteristic_uuid) {
-                info!("Setting subscribe!");
+                trace!("Setting subscribe!");
                 cb::peripheral_setnotifyvalue_forcharacteristic(
                     *p.peripheral,
                     objc::runtime::NO,
@@ -621,12 +623,12 @@ impl CoreBluetoothInternal {
                 };
             }
             adapter_msg = self.message_receiver.select_next_some() => {
-                info!("Adapter message!");
+                trace!("Adapter message!");
                 match adapter_msg {
                     CoreBluetoothMessage::StartScanning => self.start_discovery(),
                     CoreBluetoothMessage::StopScanning => self.stop_discovery(),
                     CoreBluetoothMessage::ConnectDevice(peripheral_uuid, fut) => {
-                        info!("got connectdevice msg!");
+                        trace!("got connectdevice msg!");
                         self.connect_peripheral(peripheral_uuid, fut);
                     }
                     CoreBluetoothMessage::DisconnectDevice(_peripheral_uuid, _fut) => {}
@@ -679,15 +681,26 @@ impl Drop for CoreBluetoothInternal {
 
 pub fn run_corebluetooth_thread(
     event_sender: Sender<CoreBluetoothEvent>,
-) -> Sender<CoreBluetoothMessage> {
+) -> Result<Sender<CoreBluetoothMessage>, Error> {
+    let authorization = cb::manager_authorization();
+    if authorization != CBManagerAuthorization::AllowedAlways
+        && authorization != CBManagerAuthorization::NotDetermined
+    {
+        warn!("Authorization status {:?}", authorization);
+        return Err(Error::PermissionDenied);
+    } else {
+        trace!("Authorization status {:?}", authorization);
+    }
     let (sender, receiver) = mpsc::channel::<CoreBluetoothMessage>(256);
+    // CoreBluetoothInternal is !Send, so we need to keep it on a single thread.
     thread::spawn(move || {
-        task::block_on(async move {
+        let runtime = runtime::Builder::new_current_thread().build().unwrap();
+        runtime.block_on(async move {
             let mut cbi = CoreBluetoothInternal::new(receiver, event_sender);
             loop {
                 cbi.wait_for_message().await;
             }
         })
     });
-    sender
+    Ok(sender)
 }
